class CorpusAnalysisApp {
  constructor(corpusId) {
    this.corpusId = corpusId;

    this.data = {};

    // HTML elements
    this.elements = {
      container: document.querySelector('#corpus-analysis-app-container'),
      extensionCards: document.querySelector('#corpus-analysis-app-extension-cards'),
      extensionTabs: document.querySelector('#corpus-analysis-app-extension-tabs'),
      initModal: document.querySelector('#corpus-analysis-app-init-modal')
    };
    // Materialize elements
    this.elements.m = {
      extensionTabs: M.Tabs.init(this.elements.extensionTabs),
      initModal: M.Modal.init(this.elements.initModal, {dismissible: false})
    };

    this.extensions = {};

    this.settings = {};
  }

  async init() {
    this.disableActionElements();
    this.elements.m.initModal.open();

    // Setup CQi over SocketIO connection and gather data from the CQPServer
    let cqiClient;
    let cqiCorpus;
    try {
      cqiClient = new cqi.CQiClient('/cqi_over_sio');
      let response = await cqiClient.api.socket.emitWithAck('init', this.corpusId);
      if (response.code !== 200) {throw new Error();}
      await cqiClient.connect('anonymous', '');
      cqiCorpus = await cqiClient.corpora.get(`NOPAQUE-${this.corpusId.toUpperCase()}`);
      // TODO: Don't do this hgere
      await cqiCorpus.updateDb();
    } catch (error) {
      // TODO: Currently we can only handle CQiErrors here,
      //       but we should also handle other errors.
      let errorString = `${error.code}: ${error.constructor.name}`;
      let errorsElement = this.elements.initModal.querySelector('.errors');
      let progressElement = this.elements.initModal.querySelector('.progress');
      errorsElement.innerText = errorString;
      errorsElement.classList.remove('hide');
      progressElement.classList.add('hide');
      return;
    }
    this.data.cqiClient = cqiClient;
    this.data.cqiCorpus = cqiCorpus;
    this.data.corpus = {o: cqiCorpus};  // legacy

    // Initialize extensions
    for (let extension of Object.values(this.extensions)) {extension.init();}
    for (let extensionSelectorElement of this.elements.extensionCards.querySelectorAll('.extension-selector')) {
      extensionSelectorElement.addEventListener('click', () => {
        this.elements.m.extensionTabs.select(extensionSelectorElement.dataset.target);
      });
    }

    this.enableActionElements();
    this.elements.m.initModal.close();
  }

  registerExtension(extension) {
    if (extension.name in this.extensions) {
      console.error(`Can't register extension ${extension.name}: Already registered`);
      return;
    }
    this.extensions[extension.name] = extension;
  }

  disableActionElements() {
    let actionElements = this.elements.container.querySelectorAll('.corpus-analysis-action');
    for (let actionElement of actionElements) {
      if (actionElement.nodeName === 'INPUT') {
        actionElement.disabled = true;
      } else if (actionElement.nodeName === 'SELECT') {
        actionElement.parentNode.querySelector('input.select-dropdown').disabled = true;
      } else {
        actionElement.classList.add('disabled');
      }
    }
  }

  enableActionElements() {
    let actionElements = this.elements.container.querySelectorAll('.corpus-analysis-action');
    for (let actionElement of actionElements) {
      if (actionElement.nodeName === 'INPUT') {
        actionElement.disabled = false;
      } else if (actionElement.nodeName === 'SELECT') {
        actionElement.parentNode.querySelector('input.select-dropdown').disabled = false;
      } else {
        actionElement.classList.remove('disabled');
      }
    }
  }
}
